from pyepl.locals import *
from pyepl.mechinput import *
import time
import itertools
import generate

def runCity(config, environment, clock, envstate):
    """
    """
    # create the YCEnvironment and call its run method
    YCEnvironment(config, environment, envstate).run(clock)

class YCEnvironment:
    def __init__(self, config, environment, envstate):
        """
        """
        # save the config
        self.config = config

        # save the environment components...
        self.envname = environment.name
        self.stores = environment.stores
        self.buildings = environment.buildings
        self.passengers = environment.passengers
        self.history = environment.history

        # save the environment state
        self.envstate = envstate
    def updateState(self):
        """
        """
        self.envstate.money = self.money
        #self.envstate.posorient = self.av.positionOrientation()
        #...
    def setupControls(self):
        """
        """
        # keep track of whether or not we use any left/right and forward/backward buttons...
        leftandright = False
        forwardandbackward = False

        # create forward button...
        if hasattr(self.config, "forwardButton"):
            forward_button = self.config.forwardButton
            forwardandbackward = True
        else:
            forward_button = Button() # dummy button

        # create backward button...
        if hasattr(self.config, "backwardButton"):
            backward_button = self.config.backwardButton
            forwardandbackward = True
        else:
            backward_button = Button() # dummy button

        # create left turn button...
        if hasattr(self.config, "leftButton"):
            left_button = self.config.leftButton
            leftandright = True
        else:
            left_button = Button() # dummy button

        # create right turn button...
        if hasattr(self.config, "rightButton"):
            right_button = self.config.rightButton
            leftandright = True
        else:
            right_button = Button() # dummy button

        # create forward & backward axis...
        if hasattr(self.config, "forwardBackwardAxis"):
            forwardbackward_axis = self.config.forwardBackwardAxis
        else:
            forwardbackward_axis = None

        # create left & right axis...
        if hasattr(self.config, "leftRightAxis"):
            leftright_axis = self.config.leftRightAxis
        else:
            leftright_axis = None

        # set up navigation control...
        forward_axes = []
        wheel_axes = []
        if forwardandbackward:
            forward_axes.append(ButtonAxis(forward_button,
                                           backward_button,
                                           self.config.fullForwardSpeed,
                                           self.config.fullBackwardSpeed))
        if leftandright:
            wheel_axes.append(ButtonAxis(right_button,
                                         left_button,
                                         self.config.fullTurnSpeed))
        if forwardbackward_axis:
            forward_axes.append(ScaledAxis(forwardbackward_axis, -self.config.fullBackwardSpeed, -self.config.fullForwardSpeed))
        if leftright_axis:
            wheel_axes.append(leftright_axis * self.config.fullTurnSpeed)
        self.forward = ThrottledAxis(JointAxis(*forward_axes), maxVel = self.config.maximumLinearAcceleration)
        if self.config.navigationMode == "car":
            self.turning = AxisScaledAxis(self.forward, ThrottledAxis(JointAxis(*wheel_axes), maxVel = self.config.maximumWheelSpeed))
        elif self.config.navigationMode == "doom":
            self.turning = JointAxis(*wheel_axes)
        else:
            raise ValueError, "Unknown navigation mode: %s" % self.config.navigationMode
    def setupEnvironment(self):
        """
        """
        # get the layout
        layout = generate.nextLayout(self.config, self.history, self.stores, self.buildings)

        # log the layout...
        for x in xrange(layout.getWidth()):
            for y in xrange(layout.getHeight()):
                item = layout[x, y]
                if item.type == "store":
                    self.cablog.logMessage("STORE\t%d\t%d\t%f\t%f\t%f\t%f\t%f\t%f\t%s" % (x, y, item.realpos[0], item.realpos[1], item.realpos[2], item.realsize[0], item.realsize[1], item.realsize[2], item.name))
                elif item.type == "building":
                    self.cablog.logMessage("BUILDING\t%d\t%d\t%f\t%f\t%f\t%f\t%f\t%f\t%s" % (x, y, item.realpos[0], item.realpos[1], item.realpos[2], item.realsize[0], item.realsize[1], item.realsize[2], item.name))
                elif item.type == "nullbuilding":
                    self.cablog.logMessage("NULLBUILDING\t%d\t%d\t%f\t%f\t%f" % (x, y, item.realpos[0], item.realpos[1], item.realpos[2]))
                elif item.type == "nullbuildingnullsidewalk":
                    self.cablog.logMessage("NULLBUILDINGNULLSIDEWALK\t%d\t%d\t%f\t%f\t%f" % (x, y, item.realpos[0], item.realpos[1], item.realpos[2]))

        # calculate the starting position for the avatar
        self.startpos = generate.nextStartingPosition(self.config, layout, self.history)

        # note the layout in history
        self.history.city(self.envname, layout, self.startpos)

        # build the virtual environment
        layout.buildVE(self.vr, self.config, self.storeTouch)
    def storeTouch(self, t):
        """
        """
        self.cablog.logMessage("WRONGSTORE\t%s" % t)
        self.welcome_to_text = self.config.wrongStoreText % t
    def setupPlayer(self):
        """
        """
        # set up avatar...
        self.av = self.vr.newAvatar("subject_avatar", eyeheight = self.config.eyeHeight, radius = self.config.avatarRadius, posorient = self.startpos)
        
        self.av.setControls(forward_speed = self.forward, yaw_speed = self.turning)

        # set up avatar view...
        self.eye = self.av.newEye("subject_view")
        self.eye.setFOV(self.config.FOV)
        self.video.clear() # no need to clear between frames
        self.video.show(self.eye, 0, 0)

        # set up text overlay...
        self.findapassenger = self.config.navigationOverlayFont.write(self.config.findPassengerText, self.config.navigationOverlayFontSize,
                                                                      self.config.navigationOverlayColor)
        self.goaltext = self.findapassenger
        self.lastgoaltext = self.video.show(self.goaltext, 0, 0)
        self.videowidth = self.video.getResolution()[0]
        if isinstance(self.config.moneyInitialAmount, str) and self.config.moneyInitialAmount.upper() == "CUMULATIVE":
            self.money = self.envstate.money
        else:
            self.money = self.config.moneyInitialAmount
        self.moneytext = self.config.navigationOverlayFont.write(self.config.moneyText % self.money, self.config.navigationOverlayFontSize,
                                                                 self.config.navigationOverlayColor)
        self.lastmoneytext = self.video.show(self.moneytext, self.videowidth - self.moneytext.getSize()[0], 0)
    def setupPassengers(self, dnum):
        """
        """
        passengers = generate.nextPassengersWaiting(self.config.sequence(dnum), self.history, self.passengers[dnum], dnum, self.av.positionOrientation())
        self.history.delivery(passengers)
        for n, (identity, pickup, dropoff) in enumerate(passengers):
            self.cablog.logMessage("PASSENGER\t%s\t%f\t%f\t%f\t%s" % (identity.name, pickup[0], pickup[1], pickup[2], dropoff.name))
            self.transient_objects.append(self.vr.addSprite(pickup[0], pickup[1], pickup[2],
                                                            identity.image, identity.width, identity.height))
            self.transient_objects.append(self.vr.addSphereGeom(pickup[0], pickup[1] + self.config.avatarRadius,
                                                                pickup[2], self.config.passengerRadius, permiable = True,
                                                                callback = self.pickup, cbargs = (dnum, identity, dropoff)))
    def pickup(self, dnum, passenger, destination):
        """
        """
        self.vr.removeEntity(*self.transient_objects)
        self.transient_objects = []
        self.cablog.logMessage("PICKUP\t%s" % passenger.name)
        self.history.pickup(passenger)
        self.popup_text = self.config.pickupText % destination.name
        self.popup_timeout = self.config.pickupPopupTimeOut
        self.transient_objects.append(self.vr.addBoxGeom(destination.realpos[0], destination.realpos[1], destination.realpos[2],
                                                         destination.realsize[0] + self.config.dropoffMargin, destination.realsize[1],
                                                         destination.realsize[2] + self.config.dropoffMargin, permiable = True,
                                                         callback = self.dropoff, cbargs = (dnum, passenger, destination)))
        self.goaltext = self.config.navigationOverlayFont.write(self.config.findStoreText % destination.name, self.config.navigationOverlayFontSize,
                                                                self.config.navigationOverlayColor)
    def dropoff(self, dnum, passenger, destination):
        """
        """
        self.vr.removeEntity(*self.transient_objects)
        self.transient_objects = []
        self.money += self.config.deliveryPayment
        self.cablog.logMessage("MONEY\t%f" % self.money)
        self.moneytext = self.config.navigationOverlayFont.write(self.config.moneyText % self.money, self.config.navigationOverlayFontSize,
                                                                 self.config.navigationOverlayColor)
        if dnum >= self.config.numDeliveries - 1:
            self.ycdone = True
            return
        self.cablog.logMessage("DROPOFF\t%s\t%s" % (passenger.name, destination.name))
        self.popup_text = self.config.deliveryText
        self.popup_timeout = self.config.deliveryPopupTimeOut
        self.handlePopups()
        self.must_setup_passengers = dnum + 1  # hmmm... I don't like this
        self.goaltext = self.findapassenger
    def gameLoop(self, t):
        """
        """
        thistime = now()
        remaining = thistime - self.lasttime
        if remaining >= self.config.moneyDecrementInterval:
            self.lasttime = self.lasttime + self.config.moneyDecrementInterval
            self.money -= self.config.moneyDecrementAmount
            if self.money < 0:
                self.money = 0.0
            self.cablog.logMessage("MONEY\t%f" % self.money)
            self.moneytext = self.config.navigationOverlayFont.write(self.config.moneyText % self.money, self.config.navigationOverlayFontSize,
                                                                     self.config.navigationOverlayColor)
        self.handlePopups()
        return not self.ycdone
    def handlePopups(self):
        if self.popup_on:
            if self.config.popupDismissButton.isPressed():
                instructEnd(self.config.popupDismissButton)
                self.video.unshow(self.popup_bg)
                self.video.updateScreen()
                self.popup_text = None
                self.popup_start_time = None
                self.popup_timeout = 0
                self.popup_on = False
                if not self.must_setup_passengers is None:
                    self.setupPassengers(self.must_setup_passengers)
                    self.must_setup_passengers = None
                if self.config.pauseDuringPopup:
                    self.forward.halt()
                    self.turning.halt()
                    self.av.setControls(forward_speed = self.forward, yaw_speed = self.turning)
            elif self.popup_timeout and (now() - self.popup_start_time >= self.popup_timeout):
                instructEnd()
                self.video.unshow(self.popup_bg)
                self.video.updateScreen()
                self.popup_text = None
                self.popup_start_time = None
                self.popup_timeout = 0
                self.popup_on = False
                if self.must_setup_passengers is not None:
                    self.setupPassengers(self.must_setup_passengers)
                    self.must_setup_passengers = None
                if self.config.pauseDuringPopup:
                    self.forward.halt()
                    self.turning.halt()
                    self.av.setControls(forward_speed = self.forward, yaw_speed = self.turning)
            else:
                instructStep()
        elif self.popup_text:
            self.popup_bg = self.video.show(SolidBackground(self.config.popupBackgroundColor), 0, 0)
            instructBegin(self.popup_text,
                          font = self.config.popupTextFont,
                          size = self.config.popupTextFontSize,
                          color = self.config.popupTextColor,
                          justification = self.config.popupJustification,
                          topmargin = self.config.popupTopMargin,
                          leftmargin = self.config.popupLeftMargin,
                          rightmargin = self.config.popupRightMargin)
            instructStep()
            if self.config.pauseDuringPopup:
                self.av.setControls()
            self.popup_start_time = now()
            self.popup_on = True
        elif self.welcome_to_text:
            if self.welcome_to_on:
                instructStep()
            else:
                self.popup_bg = self.video.show(SolidBackground(self.config.wrongStoreBackgroundColor), 0, 0)
                instructBegin(self.welcome_to_text,
                              font = self.config.wrongStoreTextFont,
                              size = self.config.wrongStoreTextFontSize,
                              color = self.config.wrongStoreTextColor,
                              justification = self.config.wrongStoreJustification,
                              topmargin = self.config.wrongStoreTopMargin,
                              leftmargin = self.config.wrongStoreLeftMargin,
                              rightmargin = self.config.wrongStoreRightMargin)
                self.welcome_to_on = True
            self.welcome_to_text = None
        elif self.welcome_to_on:
            instructEnd()
            self.video.unshow(self.popup_bg)
            self.video.updateScreen()
            self.welcome_to_on = False
        if not self.popup_on and self.moneytext != self.lastmoneytext:
            self.video.unshow(self.lastmoneytext)
            self.lastmoneytext = self.video.show(self.moneytext, self.videowidth - self.moneytext.getSize()[0], 0)
            self.video.updateScreen()
        if not self.popup_on and self.goaltext != self.lastgoaltext:
            self.video.unshow(self.lastgoaltext)
            self.lastgoaltext = self.video.show(self.goaltext, 0, 0)
            self.video.updateScreen()
    def setupTracks(self):
        """
        """
        # get the already loaded video track
        self.video = VideoTrack.lastInstance()

        # get the already loaded VR track
        self.vr = VRTrack.lastInstance()

        # create a custom log for the YC events
        self.cablog = LogTrack("cab")
    def run(self, clock):
        """
        """
        self.setupTracks()
        self.setupControls()
        self.lasttime = now()  # last money decrement time
        self.ycdone = False
        self.transient_objects = []
        self.must_setup_passengers = None
        self.popup_text = None
        self.popup_start_time = None
        self.popup_timeout = 0
        self.popup_on = False
        self.welcome_to_text = None
        self.welcome_to_on = False
        self.cablog.logMessage("BEGINCITY\t%s" % self.envname)
        self.setupEnvironment()
        self.setupPlayer()
        self.setupPassengers(0)
        self.video.updateScreen()
        self.cablog.logMessage("MONEY\t%f" % self.money)
        self.video.renderLoop(self.gameLoop)
        self.cablog.logMessage("ENDCITY\t%s" % self.envname)
        self.video.unshow(self.moneytext)
        self.video.unshow(self.goaltext)
        self.video.unshow(self.eye)
        self.updateState()
