from pyepl.locals import *

def getAll(environmentPool, storesfrom = (), passfrom = (), buildingsfrom = ()):
    """
    """
    # start with an empty Pool
    all = Pool()

    # search through all of the environments...
    for e in environmentPool:
        # do we want stores from this environment?
        if e.name in storesfrom:
            # ...if so, add them
            all.uniqueExtend(e.stores)

        # do we want passengers from this environment?
        if e.name in passfrom:
            # ...if so, add them
            all.uniqueExtend(e.passengers)  # FIX!

        # do we want buildings from this environment?
        if e.name in buildingsfrom:
            # ...if so, add them
            all.uniqueExtend(e.buildings)

    # return the list
    return all

def prepareVolbit(config, environmentPool):
    """
    """
    # calculate lists for getAll...
    try:
        storesFrom = config.SFPStoresFrom
    except AttributeError:
        storesFrom = []
    try:
        passengersFrom = config.SFPPassengersFrom
    except AttributeError:
        passengersFrom = []
    try:
        buildingsFrom = config.SFPBuildingsFrom
    except AttributeError:
        buildingsFrom = []

    # get the items to be presented
    all = getAll(environmentPool, storesfrom = storesFrom, passfrom = passengersFrom, buildingsfrom = buildingsFrom)

    # start a pool for the order of presentation
    order = Pool()

    # generate the random order...
    for x in xrange(config.volbitRepetitions):
        order.extend(all.sample())

    # return the order as the store front practice state
    return order

def runTimedVolbit(config, order, clock, video):
    """
    """
    # create the breakkey object
    breakkey = Key("LEFT SHIFT") & Key("S")

    # create the volbit log
    log = LogTrack("volbit")
    
    # clear the screen
    video.clear("black")

    # update the screen
    timestamp = video.updateScreen(clock)

    # log
    log.logMessage("TIMEDBEGIN", timestamp)
    
    # delay by the off time before the first presentation
    clock.delay(config.volbitOffTime)

    # for each stimulus...
    for stim in order:
        # check for the break key
        if breakkey.isPressed():
            # if it's pressed, stop presenting...
            
            # log
            log.logMessage("SKIP")

            # exit the loop
            break
        
        # show the image
        img = video.showCentered(stim.image.scale(*stim.volbitSize))

        # show the caption
        caption = video.showRelative(Text(stim.name,
                                          font = config.volbitCaptionFont,
                                          size = config.volbitCaptionFontSize,
                                          color = config.volbitCaptionColor),
                                     BELOW,
                                     img)

        # update the screen
        timestamp = video.updateScreen(clock)

        # log
        log.logMessage("ON\t%s" % stim.name, timestamp)

        # delay for the on time
        clock.delay(config.volbitOnTime)

        # clear the image and caption
        video.unshow(img, caption)

        # update the screen
        timestamp = video.updateScreen(clock)

        # log
        log.logMessage("OFF\t%s" % stim.name, timestamp)

        # delay for the off time
        clock.delay(config.volbitOffTime)

    # log
    log.logMessage("TIMEDEND", clock)

def runUntimedVolbit(config, order, clock, video):
    """
    """
    # create the breakkey object
    breakkey = Key("LEFT SHIFT") & Key("S")

    # create the volbit log
    log = LogTrack("volbit")
    
    # clear the screen
    video.clear("black")

    # update the screen
    timestamp = video.updateScreen(clock)

    # log
    log.logMessage("UNTIMEDBEGIN", timestamp)
    
    # delay by the off time before the first presentation
    clock.delay(config.volbitOffTime)

    # for each stimulus...
    for stim in order:
        # check for the break key
        if breakkey.isPressed():
            # if it's pressed, stop presenting...
            
            # log
            log.logMessage("SKIP")

            # exit the loop
            break
        
        # show the image
        img = video.showCentered(stim.image.scale(*stim.volbitSize))

        # show the caption
        caption = video.showRelative(Text(stim.name,
                                          font = config.volbitCaptionFont,
                                          size = config.volbitCaptionFontSize,
                                          color = config.volbitCaptionColor),
                                     BELOW,
                                     img)

        # update the screen
        timestamp = video.updateScreen(clock)

        # log
        log.logMessage("ON\t%s" % stim.name, timestamp)

        # wait for the continue key
        config.volbitContinueKey.wait(clock)

        # clear the image and caption
        video.unshow(img, caption)

        # update the screen
        timestamp = video.updateScreen(clock)

        # log
        log.logMessage("OFF\t%s" % stim.name, timestamp)

        # delay for the off time
        clock.delay(config.volbitOffTime)

    # log
    log.logMessage("UNTIMEDEND", clock)
